@@ -48,7 +48,6 @@ class KMeans:
       self._pynvml_exist = False
     
     self.centroids = None
-    self.num_points_in_clusters = None
 
   @staticmethod
   def cos_sim(a, b):
@@ -160,8 +159,7 @@ class KMeans:
       self.centroids = X[np.random.choice(batch_size, size=[self.n_clusters], replace=False)]
     else:
       self.centroids = centroids
-    if self.num_points_in_clusters is None:
-      self.num_points_in_clusters = torch.ones(self.n_clusters, device=device)
+    num_points_in_clusters = torch.ones(self.n_clusters, device=device)
     closest = None
     for i in range(self.max_iter):
       iter_time = time()
@@ -208,11 +206,11 @@ class KMeans:
         #   c_grad[sub_matched_clusters] = sub_prod
       error = (c_grad - self.centroids).pow(2).sum()
       if self.minibatch is not None:
-        lr = 1/self.num_points_in_clusters[:,None] * 0.9 + 0.1
-        # lr = 1/self.num_points_in_clusters[:,None]**0.1 
+        lr = 1/num_points_in_clusters[:,None] * 0.9 + 0.1
+        # lr = 1/num_points_in_clusters[:,None]**0.1 
       else:
         lr = 1
-      self.num_points_in_clusters[matched_clusters] += counts
+      num_points_in_clusters[matched_clusters] += counts
       self.centroids = self.centroids * (1-lr) + c_grad * lr
       if self.verbose >= 2:
         print('iter:', i, 'error:', error.item(), 'time spent:', round(time()-iter_time, 4))
@@ -221,9 +219,9 @@ class KMeans:
 
     # SCATTER
     if self._show:
-      if self.mode is "cosine":
+      if self.mode == "cosine":
         sim = self.cos_sim(x, self.centroids)
-      elif self.mode is "euclidean":
+      elif self.mode == "euclidean":
         sim = self.euc_sim(x, self.centroids)
       closest = sim.argmax(dim=-1)
       plt.scatter(X[:, 0].cpu(), X[:, 1].cpu(), c=closest.cpu(), marker='.', cmap='hsv')
